@@ -625,7 +625,7 @@ class ACMEdaemon(object):
                                                   n_workers=n_workers,
                                                   processes_per_worker=processes_per_worker,
                                                   mem_per_worker=mem_per_worker,
-                                                  n_workers_startup=100,
+                                                  n_workers_startup=1,
                                                   timeout=setup_timeout,
                                                   interactive=setup_interactive,
                                                   interactive_wait=120,
@@ -656,8 +656,10 @@ class ACMEdaemon(object):
         dryRunIdx, dryRunArgs, dryRunKwargs = self._dryrun_setup()
 
         # Append new dummy keyword to return before any disk-writes happen
-        for k in range(len(dryRunKwargs)):
-            dryRunKwargs[k]["memEstRun"] = True
+        # in case ACME handles results output
+        if self.out_dir is not None:
+            for k in range(len(dryRunKwargs)):
+                dryRunKwargs[k]["memEstRun"] = True
 
         # Set run-time for each job (in seconds) and allocate arrays for memory
         # consumption estimates (in GB)
@@ -702,11 +704,6 @@ class ACMEdaemon(object):
             # Compute peak memory consumption across `runTime` seconds
             memPerJob[i] = memPerSec.max()
 
-            # # Remove auto-generated files (if any were created within `runTime` seconds)
-            # if rmOutDir:
-            #     if os.path.isfile(self.kwargv["outFile"][idx]):
-            #         os.unlink(self.kwargv["outFile"][idx])
-
         # Compute aggregate average memory consumption across all runs
         memUsage = memPerJob.mean()
 
